from typing import Any, Dict

import httpx

from ...client import AuthenticatedClient
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    comment_id: str,
    property_key: str,
) -> Dict[str, Any]:
    url = "{}/rest/api/3/comment/{commentId}/properties/{propertyKey}".format(
        client.base_url, commentId=comment_id, propertyKey=property_key
    )

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _build_response(*, response: httpx.Response) -> Response[None]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    comment_id: str,
    property_key: str,
) -> Response[None]:
    kwargs = _get_kwargs(
        client=client,
        comment_id=comment_id,
        property_key=property_key,
    )

    response = httpx.delete(
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    comment_id: str,
    property_key: str,
) -> Response[None]:
    kwargs = _get_kwargs(
        client=client,
        comment_id=comment_id,
        property_key=property_key,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.delete(**kwargs)

    return _build_response(response=response)
