from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.field_metadata import FieldMetadata

T = TypeVar("T", bound="ComponentsschemasIssueTransitionFields")


@attr.s(auto_attribs=True)
class ComponentsschemasIssueTransitionFields:
    """ Details of the fields associated with the issue transition screen. Use this information to populate `fields` and `update` in a transition request. """

    additional_properties: Dict[str, FieldMetadata] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        componentsschemas_issue_transition_fields = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = FieldMetadata.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        componentsschemas_issue_transition_fields.additional_properties = additional_properties
        return componentsschemas_issue_transition_fields

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> FieldMetadata:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: FieldMetadata) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
